package cn.iocoder.yudao.module.wealth.service.receivable;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.boot.module.order.api.OrderApi;
import cn.iocoder.yudao.framework.apollo.core.event.Order.OrderOperateLogEvent;
import cn.iocoder.yudao.framework.apollo.core.vo.ApplyInfoVO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.api.customer.CustomerApi;
import cn.iocoder.yudao.module.ecw.api.customer.dto.CustomerDTO;
import cn.iocoder.yudao.module.ecw.api.region.RegionApi;
import cn.iocoder.yudao.module.ecw.api.region.dto.RegionDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import cn.iocoder.yudao.module.wealth.convert.receivable.ReceivableConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivable.ReceivableWriteOffRecordDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receivableDiscount.ReceivableDiscountDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivable.ReceivableMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receivable.ReceivableWriteOffRecordMapper;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receivableDiscount.ReceivableDiscountService;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.CurrencyAmount;
import cn.iocoder.yudao.module.wealth.vo.receivable.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.*;

/**
 * 应收款 Service 实现类
 *
 * @author 郑屹
 */
@Service
@Validated
public class ReceivableServiceImpl extends AbstractService<ReceivableMapper, ReceivableDO> implements ReceivableService {

    @Resource
    private ReceivableMapper receivableMapper;
    @Resource
    @Lazy
    private ReceiptService receiptService;
    @Resource
    private ReceivableDiscountService receivableDiscountService;
    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private ApplicationContext applicationContext;
    @Resource
    private CurrencyApi currencyApi;
    @Resource
    private CustomerApi customerApi;
    @Resource
    private RegionApi regionApi;
    @Resource
    @Lazy
    private OrderApi orderApi;
    @Resource
    private ReceivableWriteOffRecordMapper receivableWriteOffRecordMapper;
    @Resource
    private ReceiptItemMapper receiptItemMapper;

    @Override
    public Long createReceivable(ReceivableCreateReqVO createReqVO) {
        LoginUser user = SecurityFrameworkUtils.getLoginUser();
        String creator = String.valueOf(null == user ? "" : user.getId());
        // TODO 具体业务待完善
//        if (null != createReqVO.getOrderItemId()){
//
//        }
        // 插入
        ReceivableDO receivable = ReceivableConvert.INSTANCE.convert(createReqVO);
        receivable.setCreateTime(new Date());
        receivable.setCreator(creator);
        receivable.setUpdateTime(new Date());
        receivable.setUpdater(creator);
        // 设置基准币种
        receivable.setBaseCurrencyId(getDestCountryCurrencyId(createReqVO.getOrderId()));
        this.save(receivable);
        // 返回
        return receivable.getId();
    }

    @Override
    public void updateReceivable(ReceivableUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateReceivableExists(updateReqVO.getId());
        // 更新
        ReceivableDO updateObj = ReceivableConvert.INSTANCE.convert(updateReqVO);
        this.updateById(updateObj);
    }

    @Override
    public void deleteReceivable(Long id) {
        // 校验存在
        ReceivableDO receivableDO = this.getReceivable(id);
        if (receivableDO == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivableDO.getReceiptId() != null) {
            throw exception(RECEIVABLE_REMOVE_BIND_RECEIPT);
        }
        // 删除
        this.removeById(id);
    }

    @Override
    public void deleteReceivableByOrderId(Long orderId) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId));
    }

    @Override
    public void deleteReceivableByOrderIdAndFeeSource(Long orderId, Collection<Integer> feeSourceList) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>()
                .eq(ReceivableDO::getOrderId, orderId)
                .in(CollectionUtil.isNotEmpty(feeSourceList), ReceivableDO::getFeeSource, feeSourceList));
    }

    @Override
    public void deleteReceivableByOrderIdAndFeeSource(Long orderId, Integer feeSource) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId).eq(ReceivableDO::getFeeSource, feeSource));
    }

    @Override
    public void deleteReceivableByOrderItemId(Long orderItemId) {
        receivableMapper.delete(new LambdaQueryWrapperX<ReceivableDO>().eq(ReceivableDO::getOrderItemId, orderItemId));
    }

    @Override
    public ReceivableDO getReceivableByOrderItemId(Long orderItemId, Integer feeType) {
        return receivableMapper.selectOne(new LambdaQueryWrapper<ReceivableDO>()
                .eq(ReceivableDO::getOrderItemId, orderItemId)
                .eq(ReceivableDO::getFeeType, feeType)
                .eq(ReceivableDO::getFeeSource, 1));
    }

    @Override
    public void cancelReceivable(Long id) {
        ReceivableDO receivable = this.getReceivable(id);
        if (null == receivable) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivable.getDeleted()) {
            throw exception(RECEIVABLE_DELETED);
        }
        if (receivable.getState() != 0) {
            throw exception(RECEIVABLE_WRITE_OFF_ING);
        }
        this.deleteReceivable(id);
    }

    @Override
    public void updateReceivableById(ReceivableDO updateReqVO) {
        if (null == updateReqVO.getId()) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        ReceivableDO receivable = this.getReceivable(updateReqVO.getId());
        if (null == receivable) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        if (receivable.getDeleted()) {
            throw exception(RECEIVABLE_DELETED);
        }
        //收款状态（0未收款，1收款中，2已收款）
        if (receivable.getState() != 0) {
            // 取消此逻辑
            //throw exception(RECEIVABLE_WRITE_OFF_ING);
        }
        updateReqVO.setUpdateTime(new Date());
        updateReqVO.setUpdater(String.valueOf(SecurityFrameworkUtils.getLoginUserId()));
        // 更新
        this.updateById(updateReqVO);
    }

    @Override
    public void verificationReceivableIsWriteOff(Long orderId, Long orderItemId) {
        if (Objects.nonNull(orderId)) {
            // 校验整个订单的应收单是否有核销的
            long count = this.count(new LambdaQueryWrapper<ReceivableDO>().ne(ReceivableDO::getState, 0).eq(BaseDO::getDeleted, false).eq(ReceivableDO::getOrderId, orderId));
            if (count > 0) {
                throw exception(RECEIVABLE_WRITE_OFF_ING);
            }
        } else {
            // 校验订单项的应收是否核销
            if (Objects.isNull(orderItemId)) {
                throw exception(ORDER_ITEM_ID_NOT_NULL);
            }
            long count = this.count(new LambdaQueryWrapper<ReceivableDO>().ne(ReceivableDO::getState, 0).eq(BaseDO::getDeleted, false).eq(ReceivableDO::getOrderItemId, orderItemId));
            if (count > 0) {
                throw exception(RECEIVABLE_WRITE_OFF_ING);
            }
        }
    }

    private void validateReceivableExists(Long id) {
        if (this.getReceivable(id) == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
    }

    @Override
    public ReceivableDO getReceivable(Long id) {
        return this.getById(id);
    }

    @Override
    public List<ReceivableDO> getReceivableList(Collection<Long> ids) {
        return receivableMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<ReceivableDO> getReceivablePage(ReceivableQueryVO query, PageVO page) {
        return receivableMapper.selectPage(page, query);
    }

    @Override
    public PageResult<ReceivableBackVO> receivablePage(ReceivableQueryVO query, PageVO page) {
        query.setLang(I18nMessage.getLang());
        page.setField("r.id");
        IPage<ReceivableBackVO> mpPage = MyBatisUtils.buildPage(page);
        receivableMapper.receivablePage(mpPage, query);
        mpPage.getRecords().forEach(receivableBackVO -> {
            if (receivableBackVO.getDiscountTotal() == null) {
                receivableBackVO.setDiscountTotal(BigDecimal.ZERO);
            }
            receivableBackVO.setNotIncludedTaxAmount(receivableBackVO.getTotalAmount());
            receivableBackVO.setActualAmount(receivableBackVO.getNotIncludedTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
            ExchangeRateRespDTO currencyRate = currencyApi.getCurrencyRate(receivableBackVO.getCurrencyId(), receivableBackVO.getBaseCurrencyId());
            receivableBackVO.setExchangeRate(currencyRate.getCurrencyRate());
            receivableBackVO.setBaseAmount(receivableBackVO.getActualAmount().multiply(currencyRate.getCurrencyRate()));
        });
        return PageResult.of(mpPage);
    }

    @Override
    public List<WealthMoneyAmountVO> receivablePageAmount(ReceivableQueryVO query) {
        List<WealthMoneyAmountVO> voList = receivableMapper.receivablePageAmount(query);
        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();
        voList.stream().filter(vo -> allCurrency.containsKey(vo.getCurrencyId())).forEach(vo -> {
            vo.setCurrencySymbol(allCurrency.get(vo.getCurrencyId()).getFuhao());
            vo.setCurrencyName(allCurrency.get(vo.getCurrencyId()).getTitleZh());
        });
        return voList;
    }

    @Override
    public List<ReceivableBackVO> getReceivableByOrderId(Long orderId) {
        List<ReceivableBackVO> receivableBackVOList = receivableMapper.getReceivableByOrderId(orderId);
        receivableBackVOList.forEach(receivableBackVO -> {
            if (receivableBackVO.getDiscountTotal() == null) {
                receivableBackVO.setDiscountTotal(BigDecimal.ZERO);
            }
            receivableBackVO.setNotIncludedTaxAmount(receivableBackVO.getTotalAmount());
            receivableBackVO.setActualAmount(receivableBackVO.getTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
            ExchangeRateRespDTO currencyRate = currencyApi.getCurrencyRate(receivableBackVO.getCurrencyId(), receivableBackVO.getBaseCurrencyId());
            receivableBackVO.setExchangeRate(currencyRate.getCurrencyRate());
            receivableBackVO.setBaseAmount(receivableBackVO.getActualAmount().multiply(currencyRate.getCurrencyRate()));
        });
        return receivableBackVOList;
    }


    @Override
    public List<ReceivableDO> getReceivableList(ReceivableQueryVO query) {
        return receivableMapper.selectList(query);
    }


    @Override
    public Long createReceipt(Long receivableId) {
        return null;
    }

    @Override
    public List<ReceivableBackVO> receivableList(List<Long> ids) {
        return receivableMapper.receivableList(ids);
    }

    @Override
    public List<ReceivableBackVO> getReceivableListByReceiptId(Long id) {
        List<ReceivableBackVO> receivableBackVOList = receivableMapper.getReceivableBackVoListByReceiptId(id);
        receivableBackVOList.forEach(receivableBackVO -> {
            if (receivableBackVO.getDiscountTotal() == null) {
                receivableBackVO.setDiscountTotal(BigDecimal.ZERO);
            }
            receivableBackVO.setNotIncludedTaxAmount(receivableBackVO.getTotalAmount());
            receivableBackVO.setActualAmount(receivableBackVO.getTaxAmount().subtract(receivableBackVO.getDiscountTotal()));
            if (receivableBackVO.getWriteOffAmount() == null) {
                receivableBackVO.setWriteOffAmount(BigDecimal.ZERO);
            }
            receivableBackVO.setNotWriteOffAmount(receivableBackVO.getActualAmount().subtract(receivableBackVO.getWriteOffAmount()));
            if (receivableBackVO.getExchangeRate() != null) {
                receivableBackVO.setWriteOffAmountUsd(receivableBackVO.getWriteOffAmount().multiply(receivableBackVO.getExchangeRate()));
                receivableBackVO.setNotWriteOffAmountUsd(receivableBackVO.getNotWriteOffAmount().multiply(receivableBackVO.getExchangeRate()));
            } else {
                receivableBackVO.setWriteOffAmountUsd(BigDecimal.ZERO);
                receivableBackVO.setNotWriteOffAmountUsd(BigDecimal.ZERO);
            }
            if (receivableBackVO.getBaseAmount() == null) {
                receivableBackVO.setBaseAmount(BigDecimal.ZERO);
            }
            // 核销比例 = 已核销金额 / 实际金额，保留两位小数字符串
            if (receivableBackVO.getWriteOffAmount().compareTo(BigDecimal.ZERO) <= 0 || receivableBackVO.getActualAmount().compareTo(BigDecimal.ZERO) <= 0) {
                receivableBackVO.setWriteOffProportion("0.00%");
            } else {
                receivableBackVO.setWriteOffProportion(receivableBackVO.getWriteOffAmount().divide(receivableBackVO.getActualAmount(), 2, RoundingMode.HALF_UP).multiply(new BigDecimal("100")) + "%");
            }
        });
        return receivableBackVOList;
    }

    @Override
    public ReceivableBackVO getReceivableBackVo(Long id) {
        return receivableMapper.getReceivableBackVo(id);
    }

    @Override
    public List<ReceivableBackVO> getReceivableListByOrderId(Long orderId) {
        return receivableMapper.getReceivableListByOrderId(orderId);
    }

    @Override
    public List<ReceivableBackVO> getFirstReceivableListByOrderId(Long orderId) {
        return receivableMapper.getFirstReceivableListByOrderId(orderId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReceivableDiscountById(ReceivableUpdateReqVO receivableUpdateReqVO) {
        ReceivableDO receivableDO = this.getReceivable(receivableUpdateReqVO.getId());
        if (null == receivableDO) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }

        if (null != receivableDO.getReceiptId()) {
            ReceiptDO receiptDO = receiptService.getReceipt(receivableDO.getReceiptId());
            if (null != receiptDO) {
                /*if (receiptDO.getState() == 4 || receiptDO.getState() == 5 || receiptDO.getState() == 6) {
                    throw exception(RECEIPT_STATUS_NOT_EXISTS);
                }*/
            }
        }

        ReceivableDO updateReceivableDO = new ReceivableDO();
        updateReceivableDO.setId(receivableDO.getId());
        updateReceivableDO.setDiscountRemark(receivableUpdateReqVO.getDiscountRemark());
        updateReceivableDO.setDiscountTotal(receivableUpdateReqVO.getDiscountTotal());
        this.updateById(updateReceivableDO);
        //存上应收款的优惠修改记录
        ReceivableDiscountDO receivableDiscountDO = new ReceivableDiscountDO();
        receivableDiscountDO.setReceivableId(receivableDO.getId());
        receivableDiscountDO.setDiscountRemark(receivableUpdateReqVO.getDiscountRemark());
        receivableDiscountDO.setDiscountTotal(receivableUpdateReqVO.getDiscountTotal());

        AdminUserRespDTO userRespDTO = adminUserApi.getUser(SecurityFrameworkUtils.getLoginUser().getId());
        if (null != userRespDTO) {
            receivableDiscountDO.setCreatorName(userRespDTO.getNickname());
        }

        receivableDiscountService.save(receivableDiscountDO);
    }

    @Override
    public List<ReceivableOrderPayedFeeTypeVO> getPayedMoneyByFeeType(Long orderId) {

        return receivableMapper.getPayedMoneyByFeeType(orderId);
    }


    @Override
    public void orderReceivableAddReceiptLog(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            // 当前订单有应收进行反核销
            applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收加入收款单", Collections.singletonList(applyInfoVO)));
        }
    }

    @Override
    public void orderReceivableWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            List<ReceivableDO> list = receivableMapper.selectList(new LambdaQueryWrapper<ReceivableDO>().eq(ReceivableDO::getOrderId, orderId));
            // 如果没有找到订单应收，是不正常的，不需要去添加应收核销操作日志
            if (CollectionUtil.isNotEmpty(list)) {
                // 判断当前订单的应收是否有未核销完成的
                long count = list.stream().filter(r -> r.getState() != 2).count();
                if (count == 0) {
                    // 当前订单所有已生成的应收已经核销完成
                    log.error("-------------------------------");
                    applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收全部核销", Collections.singletonList(applyInfoVO)));
                } else {
                    log.error("222222222222222222222222222222222");
                    applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收部分核销", Collections.singletonList(applyInfoVO)));
                }
            }
        }
    }

    @Override
    public void orderReceivableCancelWriteOffLogAdd(Collection<Long> orderIdList, String receiptNo) {
        ApplyInfoVO applyInfoVO = new ApplyInfoVO();
        applyInfoVO.setName("收款单号");
        applyInfoVO.setNewValue(receiptNo);
        for (Long orderId : orderIdList) {
            // 当前订单有应收进行反核销
            applicationContext.publishEvent(new OrderOperateLogEvent(orderId, "财务操作", "订单应收反核销", Collections.singletonList(applyInfoVO)));
        }
    }

    @Override
    public List<ReceivableDO> receivableListByOrderId(Long orderId) {
        return receivableMapper.receivableListByOrderId(orderId);
    }


    @Override
    public List<ReceivableDO> allByOrderId(Long orderId) {
        return receivableMapper.allByOrderId(orderId);
    }

    @Override
    public List<ReceivableDO> getOrderPayList(Collection<Long> orderIdList) {
        return this.list(new LambdaQueryWrapper<ReceivableDO>()
                .ne(ReceivableDO::getState, 0)
                .eq(BaseDO::getDeleted, false)
                .in(ReceivableDO::getOrderId, orderIdList)
        );
    }

    @Override
    public void revokeDeleteReceivableById(Long receivableId) {
        receivableMapper.revokeDeleteReceivableById(receivableId);
    }

    @Override
    public CostDetailBackVO getCostDetailPage(CostDetailPageQueryVO query, PageVO page) {
        Map<Integer, CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency();

        CompletableFuture<IPage<CostDetailPageVO>> costDetailPageFuture = CompletableFuture.supplyAsync(() -> {
            IPage<CostDetailPageVO> pageInfo = new Page<>(page.getPage(), page.getRows());
            return receivableMapper.getCostDetailPage(pageInfo, query);
        });
        CompletableFuture<List<WealthMoneyAmountVO>> searchStatisticsFuture = CompletableFuture.supplyAsync(() -> receivableMapper.costDetailSearchAmount(query));
        CompletableFuture<List<WealthMoneyAmountVO>> totalStatisticsFuture = CompletableFuture.supplyAsync(() -> receivableMapper.costDetailAmount());

        try {
            CompletableFuture<Void> allFutures = CompletableFuture.allOf(costDetailPageFuture, searchStatisticsFuture, totalStatisticsFuture);
            allFutures.join();

            IPage<CostDetailPageVO> costDetailPage = costDetailPageFuture.get();
            List<WealthMoneyAmountVO> searchStatistics = searchStatisticsFuture.get();
            List<WealthMoneyAmountVO> totalStatistics = totalStatisticsFuture.get();

            if (CollectionUtil.isNotEmpty(searchStatistics)) {
                searchStatistics = searchStatistics.stream().filter(vo -> allCurrency.containsKey(vo.getCurrencyId())).collect(Collectors.toList());
                calculateStatisticalAmount(allCurrency, searchStatistics);
            }
            totalStatistics = totalStatistics.stream().filter(vo -> allCurrency.containsKey(vo.getCurrencyId())).collect(Collectors.toList());
            calculateStatisticalAmount(allCurrency, totalStatistics);

            CostDetailBackVO.CostDetailBackVOBuilder voBuilder = CostDetailBackVO.builder()
                    .searchStatistics(searchStatistics)
                    .totalStatistics(totalStatistics);
            List<CostDetailPageVO> records = costDetailPage.getRecords();

            if (CollectionUtil.isNotEmpty(records)) {
                records.forEach(vo -> {
                    vo.setCurrencySymbol(allCurrency.get(vo.getCurrencyId()).getFuhao());
                    vo.setNotIncludedTaxAmount(vo.getTotalAmount());
                    if (vo.getDiscountTotal() == null) {
                        vo.setDiscountTotal(BigDecimal.ZERO);
                    }
                    vo.setReceivableTotalAmount(vo.getTaxAmount().subtract(vo.getDiscountTotal()));
                    if (vo.getWriteOffAmount() == null) {
                        vo.setWriteOffAmount(BigDecimal.ZERO);
                    }
                    vo.setNotWriteOffAmount(vo.getReceivableTotalAmount().subtract(vo.getWriteOffAmount()));
                    if (vo.getWriteOffAmount().compareTo(BigDecimal.ZERO) <= 0 || vo.getReceivableTotalAmount().compareTo(BigDecimal.ZERO) <= 0) {
                        vo.setWriteOffProportion("0.00%");
                    } else {
                        vo.setWriteOffProportion(vo.getWriteOffAmount().divide(vo.getReceivableTotalAmount(), 4, RoundingMode.HALF_UP).multiply(new BigDecimal("100")) + "%");
                    }
                    // 存在应收明细关联的定单不存在的情况
                    if (vo.getDestinationCity() != null && vo.getDestinationCountry() != null) {
                        RegionDTO regionCity = regionApi.getRegionById(vo.getDestinationCity().longValue());
                        vo.setDestinationCityZh(regionCity.getTitleZh());
                        vo.setDestinationCityEn(regionCity.getTitleEn());
                        RegionDTO regionCountry = regionApi.getRegionById(vo.getDestinationCountry().longValue());
                        vo.setDestinationCountryZh(regionCountry.getTitleZh());
                        vo.setDestinationCountryEn(regionCountry.getTitleEn());
                    }
                });
                return voBuilder
                        .list(records)
                        .total(costDetailPage.getTotal())
                        .rows(costDetailPage.getSize())
                        .page(costDetailPage.getCurrent())
                        .pages(costDetailPage.getPages())
                        .build();
            }
            return voBuilder
                    .list(Collections.EMPTY_LIST)
                    .total(costDetailPage.getTotal())
                    .rows(costDetailPage.getSize())
                    .page(costDetailPage.getCurrent())
                    .pages(costDetailPage.getPages())
                    .build();
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException("查询异常");
        }
    }

    /**
     * 实际金额 = 含税金额 - 优惠金额
     * 未核销金额 = 实际金额 - 核销金额
     * TODO 暂定 不含税金额 = 总金额
     *
     * @param allCurrency
     * @param statistics
     */
    private void calculateStatisticalAmount(Map<Integer, CurrencyRespDTO> allCurrency, List<WealthMoneyAmountVO> statistics) {
        statistics.forEach(vo -> {
            vo.setCurrencySymbol(allCurrency.get(vo.getCurrencyId()).getFuhao());
            vo.setCurrencyName(allCurrency.get(vo.getCurrencyId()).getTitleZh());
            if (vo.getWriteOffAmount() == null) {
                vo.setWriteOffAmount(BigDecimal.ZERO);
            }
            if (vo.getDiscountTotal() == null) {
                vo.setDiscountTotal(BigDecimal.ZERO);
            }
            vo.setNotIncludedTaxAmount(vo.getTotalAmount());
            vo.setReceivableTotalAmount(vo.getTaxAmount().subtract(vo.getDiscountTotal()));
            vo.setNotWriteOffAmount(vo.getReceivableTotalAmount().subtract(vo.getWriteOffAmount()));
        });
    }

    @Override
    public PageResult<BatchGenReceiptPageVO> batchGenReceiptPage(BatchGenReceiptPageQueryVO query, PageVO page) {
        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        List<BatchGenReceiptPageVO> batchGenReceiptPageVOS = receivableMapper.batchGenReceiptPage(start, size, query);
        int total = receivableMapper.countBatchGenReceiptPage(query);
        batchGenReceiptPageVOS.forEach(vo -> {
            List<CurrencyAmount> receivableAmount = receivableMapper.calculateReceivableAmountByOrderId(vo.getOrderId());
            vo.setReceivableAmount(receivableAmount);
            CustomerDTO customer;
            if (vo.getDrawee().equals(1)) {
                customer = customerApi.getCustomerInfo(vo.getConsignorId());
                vo.setPayerId(vo.getConsignorId());
            } else {
                customer = customerApi.getCustomerInfo(vo.getConsigneeId());
                vo.setPayerId(vo.getConsigneeId());
            }
            vo.setCustomerName(customer.getName());
            vo.setCustomerNo(customer.getNumber());
            vo.setPayerName(vo.getPayerName());
        });
        return new PageResult<>(batchGenReceiptPageVOS, total, page.getRows(), page.getPage(), (total + page.getRows() - 1) / page.getRows());
    }

    @Override
    public Long getDestCountryCurrencyId(Long orderId) {
        Long countryId = orderApi.getDestCountryByOrderId(orderId);
        if (countryId == null) {
            return null;
        }
        RegionDTO regionById = regionApi.getRegionById(countryId);
        return Long.parseLong(regionById.getImportCurrency5());
    }

    @Override
    public void createWriteOffRecord(ReceivableWriteOffReqVO receivableWriteOffReqVO, ReceiptItemDO receiptItem) {
        ReceivableDO receivableDO = receivableMapper.selectById(receivableWriteOffReqVO.getReceivableId());
        if (receivableDO == null) {
            throw exception(RECEIVABLE_NOT_EXISTS);
        }
        ReceivableWriteOffRecordDO writeOffRecord = ReceivableWriteOffRecordDO.builder()
                .receivableId(receivableWriteOffReqVO.getReceivableId())
                .writeOffAmount(receivableWriteOffReqVO.getWriteOffAmount())
                .receiptItemId(receiptItem.getId())
                .build();
        receivableWriteOffRecordMapper.insert(writeOffRecord);
    }

    @Override
    public ReceivableWriteOffRecordDO getReceivableWriteOffRecord(Long receivableId, Long receiptItemId) {
        return receivableWriteOffRecordMapper.selectOne("receivable_id", receivableId, "receipt_item_id", receiptItemId);
    }

    @Override
    public List<ReceivableInItemVO> getListForCreateReceiptItem(Long receiptId) {
        List<ReceivableDO> receivableDOS = this.selectList("receipt_id", receiptId);
        List<ReceivableInItemVO> receivableInItemVOS = ReceivableConvert.INSTANCE.convertToItemVOList(receivableDOS);
        for (ReceivableInItemVO receivableInItemVO : receivableInItemVOS) {
            receivableInItemVO.setActualAmount(receivableInItemVO.getTaxAmount().subtract(receivableInItemVO.getDiscountTotal() == null ? BigDecimal.ZERO : receivableInItemVO.getDiscountTotal()));
            BigDecimal totalWriteOffAmount = receivableWriteOffRecordMapper.getTotalAmountForReceivable(receivableInItemVO.getId());
            if (totalWriteOffAmount == null) {
                totalWriteOffAmount = BigDecimal.ZERO;
            }
            receivableInItemVO.setWriteOffAmountUsd(totalWriteOffAmount);
            receivableInItemVO.setNotWriteOffAmountUsd(receivableInItemVO.getBaseAmount().subtract(totalWriteOffAmount));
        }
        receivableInItemVOS.sort(Comparator.comparing(ReceivableInItemVO::getNotWriteOffAmountUsd));
        return receivableInItemVOS;
    }


    @Override
    public List<ReceivableInItemVO> getReceivableByReceiptItem(Long receiptItemId) {
        ReceiptItemDO receiptItemDO = receiptItemMapper.selectById(receiptItemId);
        List<ReceivableDO> receivableDOS = this.selectList("receipt_id", receiptItemDO.getReceiptId());
        List<ReceivableInItemVO> receivableInItemVOS = ReceivableConvert.INSTANCE.convertToItemVOList(receivableDOS);
        for (ReceivableInItemVO receivableInItemVO : receivableInItemVOS) {
            receivableInItemVO.setActualAmount(receivableInItemVO.getTaxAmount().subtract(receivableInItemVO.getDiscountTotal() == null ? BigDecimal.ZERO : receivableInItemVO.getDiscountTotal()));
            ReceivableWriteOffRecordDO receivableWriteOffRecordDO = this.getReceivableWriteOffRecord(receivableInItemVO.getId(), receiptItemId);
            if (null == receivableWriteOffRecordDO) {
                receivableInItemVO.setWriteOffAmount(BigDecimal.ZERO);
            } else {
                receivableInItemVO.setWriteOffAmount(receivableWriteOffRecordDO.getWriteOffAmount());
            }
            BigDecimal totalWriteOffAmount = receivableWriteOffRecordMapper.getTotalAmountForReceivable(receivableInItemVO.getId());
            if (totalWriteOffAmount == null) {
                receivableInItemVO.setWriteOffAmountUsd(BigDecimal.ZERO);
            } else {
                receivableInItemVO.setWriteOffAmountUsd(totalWriteOffAmount.subtract(receivableInItemVO.getWriteOffAmount()));
            }
            receivableInItemVO.setNotWriteOffAmountUsd(receivableInItemVO.getBaseAmount().subtract(receivableInItemVO.getWriteOffAmountUsd()));
        }
        receivableInItemVOS.sort(Comparator.comparing(ReceivableInItemVO::getNotWriteOffAmountUsd));
        return receivableInItemVOS;
    }

    @Override
    public ReceivablePageResult<BatchGenReceiptReceivablePageVO> batchGenReceiptReceivablePage(BatchGenReceiptReceivablePageQueryVO query, PageVO page) {
        IPage<BatchGenReceiptReceivablePageVO> mpPage = MyBatisUtils.buildPage(page);
        receivableMapper.batchGenReceiptReceivablePage(mpPage, query);
        ReceivablePageResult<BatchGenReceiptReceivablePageVO> pageResult = ReceivablePageResult.of(mpPage);
        List<CurrencyAmount> totalActualAmount = receivableMapper.batchGenReceiptReceivablAmount(query);
        pageResult.setTotalActualAmount(totalActualAmount);
        return pageResult;
    }

    @Override
    public void deleteWriteOffRecord(ReceiptItemDO receiptItemDO) {
        receivableWriteOffRecordMapper.deleteWriteOffRecord(receiptItemDO.getId());
    }
}